class Constraint implements IUIComponentConstraint {
  public get movable(): boolean {
    return true;
  }

  public get removable(): boolean {
    return true;
  }

  public get copyable(): boolean {
    return true;
  }

  public get extremity(): boolean {
    return true;
  }

  public acceptableChild(): boolean {
    return false;
  }

  public acceptableParent(parent: ParentUIElement): boolean {
    if (parent.clazz !== 'UIElement') return false;

    return true;
  }
}

type PropertyDefinition = {
  json: UniquePropertyDefinitionType;
};

const uniquePropertyDefinition: PropertyDefinition & IUniquePropertyDefinition = {
  json: {
    displayName: 'json',
    definition: {
      acceptStatic: false,
      rerender: true,
    },
    type: 'string',
    value: '',
  }
};

const commonPropertyDefinition: ICommonPropertyDefinition = {};

export class CookBook181352Highcharts implements IUIElementCore {
  private _chart: Highcharts.ChartObject;

  public get elementTypeName(): string {
    return CookBook181352Highcharts.name;
  }

  public get displayType(): UIElementDisplayType {
    return 'CANVAS';
  }

  public get constraintClass(): UIComponentConstraintClass {
    return Constraint;
  }

  public get uniquePropertyDefinition(): IUniquePropertyDefinition {
    return uniquePropertyDefinition;
  }

  public get commonPropertyDefinition(): ICommonPropertyDefinition {
    return commonPropertyDefinition;
  }

  public get dependElements(): UIElementCoreClass[] {
    return [];
  }

  public get wrapperClass(): UIElementWrapperClass {
    return 'SimpleUIElement';
  }

  public createElement(): IHTMLElementBuilder {
    return window.imHichee.HTMLElementBuilder.createElement('div', HTMLDivElement);
  }

  public updateElement(): void {
    // DO NOTHING
  }

  public destroyElement(): void {
    // DO NOTHING
  }

  public createChildren(
    self: IUIElement,
    container: IUIContainer,
    properties: IUIElementPropertyAccessor<IUniquePropertyDefinition & ICommonPropertyDefinition>
  ): IUIElement[] {
    // Set the default size of the element.
    if (container.designMode) {
      const width = self.properties.getProperty('styleWidth');
      if (!width.value.isValid()) {
        width.value.raw = window.imHichee.ArgumentFactory.createAsString(container, '600px');
      }

      const height = self.properties.getProperty('styleHeight');
      if (!height.value.isValid()) {
        height.value.raw = window.imHichee.ArgumentFactory.createAsString(container, '400px');
      }
    }

    return [];
  }

  public updateChildren(
    children: IUIElement[],
    self: IUIElement,
    container: IUIContainer,
    properties: IUIElementPropertyAccessor<IUniquePropertyDefinition & ICommonPropertyDefinition>
  ): IUIElement[] {
    const jsonProperty = container.parameters.toPrimitive(properties.getProperty('json'));
    const json = jsonProperty && typeof jsonProperty === 'object' ? JSON.parse(JSON.stringify(jsonProperty)) : {};

    window.imHichee.BrowserHelper.nextTick(() => {
      if (this._chart) this._chart.destroy();
      this._chart = window.Highcharts.chart(self.builder.tag, json);
    });

    return children;
  }

  public clonedChildren(
    children: IUIElement[],
    self: IUIElement,
    container: IUIContainer,
    properties: IUIElementPropertyAccessor<IUniquePropertyDefinition & ICommonPropertyDefinition>
  ): IUIElement[] {
    return children;
  }
}
